//////////////////////////////////////
// Internet TicTacToe Project
// CS571, Spring 2002
//
// by Rendong Bai <rbai2@uky.edu>
//    Yongwei Wang <ywang7@uky.edu>
//    Lengning Liu <lliu1@uky.edu>
//
// Dept. of Computer Science
// University of Kentucky   
//////////////////////////////////////

//////////////////////////////////////
// This module was written by
// Lengning Liu
//////////////////////////////////////

#include "array.h"

template <class Type>
const int CArray<Type>::DEFAULT_SIZE;

template <class Type>
CArray<Type>::CArray(const CArray & array)
{
	m_iTotalNum = m_iLength = 0;
	*this = array;
}

template <class Type>
CArray<Type>::CArray(const Type array[], int length)
{
	newArray(length);
	append(array, length);
}

template <class Type>
CArray<Type>::CArray(int size)
{
	newArray(size);
}

template <class Type>
CArray<Type>::~CArray()
{
	clean();
}

template <class Type>
void CArray<Type>::clean()
{
	if ( m_iTotalNum > 0 )
		delete [] m_pArray;
	m_iTotalNum = m_iLength = 0;
	m_pArray = NULL;
}

template <class Type>
void CArray<Type>::partition(int low, int high,
				    int & newLow, int & newHigh)
{
	int pivotValue = m_pArray[low];
	int oldHigh = high;
	
	while ( low < high )
	{
		while ( low <= high && m_pArray[low] < pivotValue )
			low++;
		while ( high >= low && m_pArray[high] >= pivotValue )
			high--;
		if ( low > high )
			break;

		Type tmp = m_pArray[low];
		m_pArray[low] = m_pArray[high];
		m_pArray[high] = tmp;
	}
	newHigh = low - 1;

	high = oldHigh;
	while ( low < high )
	{
		while ( low <= high && m_pArray[low] <= pivotValue )
			low++;
		while ( high >= low && m_pArray[high] > pivotValue )
			high--;
		if ( low > high )
			break;

		Type tmp = m_pArray[low];
		m_pArray[low] = m_pArray[high];
		m_pArray[high] = tmp;
	}
	newLow = high + 1;
}

template <class Type>
void CArray<Type>::quickSort(int low, int high)
{
	int newLow, newHigh;

	if ( high > low )
	{
		partition(low, high, newLow, newHigh);
		quickSort(low, newHigh);
		quickSort(newLow, high);
	}
}

template <class Type>
void CArray<Type>::empty()
{
	m_iLength = 0;
//	memset(m_pArray, 0, m_iTotalNum*sizeof(Type));
}

template <class Type>
bool CArray<Type>::newArray(int size)
{
	if ( size > 0 )
	{
		m_iLength = 0;
		m_iTotalNum = size;
		m_pArray = new Type[m_iTotalNum];
		if ( m_pArray == NULL )
			throw CArrayException("newArray: out of memory");
//		memset(m_pArray, 0, m_iTotalNum*sizeof(Type));
		return true;
	}
	else
	{
		m_iLength = 0;
		m_iTotalNum = 0;
		m_pArray = NULL;
		return false;
	}
}

template <class Type>
void CArray<Type>::enlargeArray()
{
	if ( m_iTotalNum <= 0 )
	{
		newArray();
	}
	else
	{
		Type * temp;
		temp = new Type[m_iTotalNum * 2];
		if ( temp == NULL )
			throw CArrayException("enlargeArray: out of memory");
		
//		memset(temp, 0, m_iTotalNum * 2);
		memcpy(temp, m_pArray, m_iTotalNum*sizeof(Type));
		delete [] m_pArray;

		m_iTotalNum *= 2;

		m_pArray = temp;
	}
}

template <class Type>
int CArray<Type>::shrinkArray()
{
	if ( m_iLength > 0 )
	{
		if ( m_iLength < m_iTotalNum )
		{
			Type * temp;
			temp = new Type[m_iLength];
			if ( temp == NULL )
				throw CArrayException("shrinkArray: out of memory");
			
			memcpy(temp, m_pArray, m_iLength*sizeof(Type));
			delete [] m_pArray;
			m_pArray = temp;
		}
		else
			m_iTotalNum = m_iLength;		
	}
	else if ( m_iTotalNum > 0 )
	{
		delete [] m_pArray;
		m_iTotalNum = m_iLength = 0;		
	}
	else
		m_iTotalNum = m_iLength = 0;

	return m_iLength;
}

template <class Type>
void CArray<Type>::setLength(int len)
{
	while ( len > m_iTotalNum )
		enlargeArray();

	m_iLength = len;
}

template <class Type>
int CArray<Type>::addNoconst(Type & item)
{
	if ( m_iLength >= m_iTotalNum )
	{
		enlargeArray();
	}

	m_pArray[m_iLength].assignNoconst(item);
	m_iLength++;

	return m_iLength-1;
}

template <class Type>
int CArray<Type>::addNoDup(const Type & item)
{
	if ( m_iLength >= m_iTotalNum )
	{
		enlargeArray();
	}

	if ( !hasIt(item) )
	{
		m_pArray[m_iLength] = item;
		m_iLength++;
	}
	
	return m_iLength-1;
}

template <class Type>
int CArray<Type>::add(const Type & item)
{
	if ( m_iLength >= m_iTotalNum )
	{
		enlargeArray();
	}

	m_pArray[m_iLength] = item;
	m_iLength++;

	return m_iLength-1;
}

template <class Type>
int CArray<Type>::put(int index, const Type & item)
{
	while ( index >= m_iTotalNum )
	{
		enlargeArray();
	}

	m_pArray[index] = item;

	return index;
}

template <class Type>
int CArray<Type>::append(const CArray<Type> & array)
{
	for ( int i = 0; i < array.m_iLength; i++ )
		add(array[i]);
	
	return m_iLength-1;
}

template <class Type>
int CArray<Type>::append(const Type array[], int length)
	throw (CArrayException)
{
	if ( length < 0 )
		throw CArrayException("append/2: invalid length in append()");
	
	for ( int i = 0; i < length; i++ )
		add(array[i]);
	
	return m_iLength-1;
}

template <class Type>
int CArray<Type>::fastRemove(int index)
	throw (CArrayException)
{
	if ( index < 0 || index >= m_iLength )
		throw CArrayException("fastRemove: index out of bound");

	m_pArray[index] = m_pArray[m_iLength - 1];
	m_iLength--;

	return m_iLength;
}

template <class Type>
const CArray<Type> & CArray<Type>::assign(const Type array[],
							int length)
	throw (CArrayException)
{
	if ( length < 0 )
		throw CArrayException("assign: invalid length in assign()");
	
	clean();
	m_iLength = 0;
	m_iTotalNum = length;
	newArray(m_iTotalNum);
	for ( int i = 0; i < length; i++ )
		add(array[i]);
	
	return *this;	
}

template <class Type>
bool CArray<Type>::hasIt(const Type &item) const
{
	for ( int i = 0; i < m_iLength; i++ )
		if ( m_pArray[i] == item )
			return true;
	return false;
}

template <class Type>
void CArray<Type>::sort()
{
	quickSort(0, m_iLength-1);
}

template <class Type>
const CArray<Type> & CArray<Type>::assignNoconst
(CArray<Type> & array)
{
	clean();
	m_iLength = 0;
	m_iTotalNum = array.m_iTotalNum;
	newArray(m_iTotalNum);
	for ( int i = 0; i < array.m_iLength; i++ )
		addNoconst(array.m_pArray[i]);
	
	return *this;
}

template <class Type>
const CArray<Type> & CArray<Type>::operator =
(const CArray<Type> & array)
{
	clean();
	m_iLength = 0;
	m_iTotalNum = array.m_iTotalNum;
	newArray(m_iTotalNum);
	for ( int i = 0; i < array.m_iLength; i++ )
		add(array.m_pArray[i]);
	
	return *this;
}

template <class Type>
inline Type & CArray<Type>::operator [] (int index) const
	throw (CArrayException)
{
	if ( index < 0 || index >= m_iLength )
		throw CArrayException("[]: index out of bound");
	
	return m_pArray[index];
}

template <class Type>
inline bool CArray<Type>::operator ==
(const CArray<Type> & array) const
{
	if ( m_iLength != array.m_iLength )
		return false;

	for ( int i = 0; i < m_iLength; i++ )
		if ( m_pArray[i] != array.m_pArray[i] )
			return false;
	return true;
}

template <class Type>
inline bool CArray<Type>::operator !=
(const CArray<Type> & array) const
{
	return !(*this == array);
}

template <class Type>
void CArray<Type>::dump() const
{

}
