////////////////////////////////////////
// Package for building a dynamic array
// by Lengning Liu <lliu1@cs.uky.edu>
////////////////////////////////////////

#ifndef __ARRAY_H__
#define __ARRAY_H__

#include "myexception.h"

template <class Type>
class CArray
{
 protected:
	static const int DEFAULT_SIZE = 128;

	Type * m_pArray;
	int m_iLength;
	int m_iTotalNum;
	int m_iCurrent;
	
	void enlargeArray();
	void clean();
	void partition(int low, int high, int & newLow, int & newHigh);
	void quickSort(int low, int high);
	
 public:
	CArray() { newArray(0); }
	CArray(const CArray & array);
	CArray(const Type array[], int length);
	CArray(int size);
	~CArray();
	bool newArray(int size = DEFAULT_SIZE);
	int shrinkArray();
	void empty();

	void setLength(int len);
	int getLength() const { return m_iLength; }
	int getSize() const { return m_iTotalNum; }
	
	int addNoconst(Type & item);
	int addNoDup(const Type & item);
	int add(const Type & item);
	int put(int index, const Type & item);
	int append(const CArray & array);
	int append(const Type array[], int length)
		throw (CArrayException);

	int fastRemove(int index)
		throw (CArrayException);

	bool isEnd() const { return m_iCurrent == m_iLength-1; }
	void rewind() { m_iCurrent = 0; }
	bool next() 
	{
		if ( !isEnd() )
		{
			m_iCurrent++;
			return true;
		}
		else
			return false;
	}
	Type & getCurrent() const
		throw (CArrayException)
	{
		if ( m_iCurrent < 0 || m_iCurrent >= m_iLength )
			throw CArrayException("'current' out of bound");
		
		return m_pArray[m_iCurrent];
	}
	int getCurrentIndex() const
		throw (CArrayException)
	{
		if ( m_iCurrent < 0 || m_iCurrent >= m_iLength )
			throw CArrayException("'current' out of bound");
		
		return m_iCurrent;
	}
	
	Type * toArray() const { return m_pArray; }

	bool hasIt(const Type & item) const;
	void sort();
	
	const CArray & assign(const Type array[], int length)
		throw (CArrayException);
	const CArray & assignNoconst(CArray & array);
	const CArray & operator = (const CArray & array);
	Type & operator [] (int index) const
		throw (CArrayException);		
	bool operator == (const CArray<Type> & array) const;
	bool operator != (const CArray<Type> & array) const;

	void dump() const;
};

#endif
