#ifndef __HASH_H__
#define __HASH_H__

#include <list>
#include "myexception.h"

template <class DataType>
class CHash
{
 protected:
	struct SPair
	{
		int m_iKey;
		DataType * m_pData;

		SPair() {};
		SPair(int key, DataType * data)
	        {
			m_iKey = key;
			m_pData = data;
		}
	};

	static const unsigned int SIZE = 701;

 protected:
	int m_iSize;
	list<SPair> *m_pTable;

	int hash(int key) const
	{
		return key % m_iSize;
	}
	
 public:
	CHash() { m_iSize = SIZE; setSize(m_iSize); };
	CHash(int size)
	{
		setSize(size);
	}
	
	~CHash()
	{
		if ( m_iSize > 0 )
		{
			delete [] m_pTable;
		}
	}
	
	void setSize(int size)
	{
		if ( size < 0 ) return;
		m_iSize = size;
		m_pTable = new list<SPair>[m_iSize];
		if ( m_pTable == NULL )
			throw CHashException("setSize: cannot allocate table");
	}

	// Note:
	// No error checking in the following functions. Make sure your
	// hash table has already created successfully before using the
	// following functions.
	void insert(int key, DataType * data)
	{
		int pos = hash(key);
		SPair keydata(key, data);
		
		m_pTable[pos].push_front(keydata);
	}

	DataType * get(int key)
	{
		int pos = hash(key);
		
		typename list<SPair>::iterator chain;
		for ( chain = m_pTable[pos].begin();
		      chain != m_pTable[pos].end();
		      chain++ )
		{
			if ( chain->m_iKey == key )
			{
				return chain->m_pData;
			}
		}

		return NULL;
	}

	const DataType * constGet(int key) const
	{
		int pos = hash(key);
		
		typename list<SPair>::iterator chain;
		for ( chain = m_pTable[pos].begin();
		      chain != m_pTable[pos].end();
		      chain++ )
		{
			if ( chain->m_iKey == key )
			{
				return chain->m_pData;
			}
		}

		return NULL;
	}

	DataType * operator[](int key)
	{
		return get(key);
	}

	const CHash & operator = (const CHash & other)
	{
		m_iSize = other.m_iSize;
		for ( int i = 0; i < m_iSize; i++ )
		{
			m_pTable[i] = other.m_pTable[i];
		}

		return *this;
	}
};
/*
template <class DataType>
class CFixedHash : public CHash<DataType>
{
 protected:
	static const unsigned int LIMIT = 20;
	
 public:
	CFixedHash() { m_iSize = SIZE; setSize(m_iSize); };
	CFixedHash(int size)
	{
		setSize(size);
	}
	
	~CFixedHash()
	{
		if ( m_iSize > 0 )
		{
//			delete [] m_pTable;
		}
	}
	
	// Note:
	// No error checking in the following functions. Make sure your
	// hash table has already created successfully before using the
	// following functions.
	void insert(int key, DataType * data)
	{
		int pos = hash(key);
		typename CHash<DataType>::SPair keydata(key, data);

		typename list<typename CHash<DataType>::SPair>::iterator chain;
		for ( chain = m_pTable[pos].begin();
		      chain != m_pTable[pos].end();
		      chain++ )
		{
			if ( chain->m_iKey == key )
			{
				delete chain->m_pData;
				m_pTable[pos].erase(chain);
				break;
			}
		}

		if ( m_pTable[pos].size() > LIMIT )
		{
			delete m_pTable[pos].back().m_pData;
			m_pTable[pos].pop_back();
		}
		
		m_pTable[pos].push_front(keydata);
	}

	// does not check duplicates
	void fastInsert(int key, DataType * data)
	{
		int pos = hash(key);
		typename CHash<DataType>::SPair keydata(key, data);

		if ( m_pTable[pos].size() > LIMIT )
		{
			delete m_pTable[pos].back().m_pData;
			m_pTable[pos].pop_back();
		}
		
		m_pTable[pos].push_front(keydata);
	}

	const CFixedHash & operator = (const CFixedHash & other)
	{
		m_iSize = other.m_iSize;
		for ( int i = 0; i < m_iSize; i++ )
		{
			m_pTable[i] = other.m_pTable[i];
		}

		return *this;
	}
};
*/
#endif
