//////////////////////////////////////
// Internet TicTacToe Project
// CS571, Spring 2002
//
// by Rendong Bai <rbai2@uky.edu>
//    Yongwei Wang <ywang7@uky.edu>
//    Lengning Liu <lliu1@uky.edu>
//
// Dept. of Computer Science
// University of Kentucky   
//////////////////////////////////////

//////////////////////////////////////
// This module was written by
// Lengning Liu
//////////////////////////////////////

#include "list.h"

////////////////////////
// CList<Type> class
////////////////////////
template <class Type>
CList<Type>::CList()
	throw (CListException)
{
	m_pHead = new TBuffer;
	if ( m_pHead == NULL )
		throw CListException("out of memory");
	m_pHead->m_pNext = NULL;
	m_pCurr = m_pHead;
	m_pTail = m_pHead;
	m_iLength = 0;
}

template <class Type>
CList<Type>::CList(const CList<Type> & list)
{
	m_pHead = new TBuffer;
	if ( m_pHead == NULL )
		throw CListException("out of memory");
	m_pHead->m_pNext = NULL;
	m_pCurr = m_pHead;
	m_pTail = m_pHead;
	m_iLength = 0;

	*this = list;
}

template <class Type>
CList<Type>::~CList()
{
	empty();
	delete m_pHead;
}

template <class Type>
void CList<Type>::empty()
{
	if ( m_pHead == NULL )
	{
		m_pHead = new TBuffer;
		if ( m_pHead == NULL )
			throw CListException("out of memory");
		m_pHead->m_pNext = NULL;
		m_pCurr = m_pHead;
		m_pTail = m_pHead;
		m_iLength = 0;
		return;
	}

	if ( isEmpty() )
		return;

	TBuffer * pnext;
	m_pCurr = m_pHead;
	pnext = m_pCurr->m_pNext;
	
	for ( int i = 0; i < m_iLength; i++ )
	{
		m_pCurr = pnext;
		pnext = m_pCurr->m_pNext;
		delete m_pCurr;
	}
	
	m_pHead->m_pNext = NULL;
	m_pCurr = m_pHead;
	m_pTail = m_pHead;
	m_iLength = 0;
}

template <class Type>
inline bool CList<Type>::isEmpty() const { return m_iLength <= 0; }

template <class Type>
bool CList<Type>::hasIt(const Type & buffer, unsigned  int offset) const
{
	TBuffer * pbuf = m_pHead->m_pNext;
	
	for ( int i = 0; i < m_iLength; i++ )
	{
		if ( pbuf->m_daBuffer == buffer &&
		     pbuf->m_uiOffset == offset )
			return true;
		pbuf = pbuf->m_pNext;
	}
	
	return false;
}

template <class Type>
void CList<Type>::addNoconst(Type & buffer, unsigned int offset)
	throw (CListException)
{
	TBuffer * pbuf = new TBuffer;

	if ( pbuf == NULL )
		throw CListException("out of memory");
	pbuf->m_daBuffer.assignNoconst(buffer);
	pbuf->m_uiOffset = offset;
	
	pbuf->m_pNext = NULL;
	m_pTail->m_pNext = pbuf;
	m_pTail = m_pTail->m_pNext;
	m_iLength++;
}

template <class Type>
void CList<Type>::add(const Type & buffer, unsigned int offset)
	throw (CListException)
{
	TBuffer * pbuf = new TBuffer;

	if ( pbuf == NULL )
		throw CListException("out of memory");
	pbuf->m_daBuffer = buffer;
	pbuf->m_uiOffset = offset;
	
	pbuf->m_pNext = NULL;
	m_pTail->m_pNext = pbuf;
	m_pTail = m_pTail->m_pNext;
	m_iLength++;
}

template <class Type>
void CList<Type>::append(const CList<Type> & list)
	throw (CListException)
{
	const TBuffer * pbuf = list.m_pHead->m_pNext;
	for ( int i = 0; i < list.getLength(); i++ )
	{
		add(pbuf->m_daBuffer, pbuf->m_uiOffset);
		pbuf = pbuf->m_pNext;
	}
}

template <class Type>
void CList<Type>::remove(const Type & buffer, unsigned int offset)
	throw (CListException)
{
	TBuffer * pbuf = m_pHead->m_pNext;
	TBuffer * pprev = m_pHead;
	int i;
	
	for ( i = 0; i < m_iLength; i++ )
	{
		if ( pbuf->m_daBuffer == buffer &&
		     pbuf->m_uiOffset == offset )
			break;
		pprev = pbuf;
		pbuf = pbuf->m_pNext;
	}

	if ( i >= m_iLength )
		return;

	pprev->m_pNext = pbuf->m_pNext;
	if ( m_pTail == pbuf )
		m_pTail = pprev;
	if ( m_pCurr == pbuf )
		m_pCurr = pprev;
	
	delete pbuf;
	m_iLength--;
}

template <class Type>
void CList<Type>::removeAtIndex(int index)
	throw (CListException)
{
	if ( index < 0 || index > m_iLength )
		return;
	
	TBuffer * pbuf = m_pHead->m_pNext;
	TBuffer * pprev = m_pHead;
	
	for ( int i = 0; i < m_iLength; i++ )
	{
		if ( i == index )
			break;
		pprev = pbuf;
		pbuf = pbuf->m_pNext;
	}

	pprev->m_pNext = pbuf->m_pNext;
	if ( m_pTail == pbuf )
		m_pTail = pprev;
	if ( m_pCurr == pbuf )
		m_pCurr = pprev;
	
	delete pbuf;	
	m_iLength--;
}

template <class Type>
inline int CList<Type>::getLength() const { return m_iLength; }

template <class Type>
inline bool CList<Type>::isHead() const { return m_pCurr == m_pHead; }

template <class Type>
inline bool CList<Type>::isEnd() const { return m_pCurr->m_pNext == NULL; }

template <class Type>
inline Type & CList<Type>::getCurrent()  const
	throw (CListException)
{
	if ( isEnd() )
		throw CListException("empty list");
	return m_pCurr->m_pNext->m_daBuffer;
}

template <class Type>
inline unsigned int CList<Type>::getCurrentOffset() const
	throw (CListException)
{
	if ( isEnd() )
		throw CListException("empty list");

	return m_pCurr->m_pNext->m_uiOffset;
}

template <class Type>
inline void CList<Type>::setCurrent(const Type & buffer)
	throw (CListException)
{
	if ( isEnd() )
		throw CListException("end of the list");
	m_pCurr->m_pNext->m_daBuffer = buffer;
}

template <class Type>
void CList<Type>::insertAfter(const Type & buffer, unsigned int offset)
	throw (CListException)
{
	TBuffer * pbuf = new TBuffer;

	if ( pbuf == NULL )
		throw CListException("out of memory");
	pbuf->m_daBuffer = buffer;
	pbuf->m_uiOffset = offset;
	
	pbuf->m_pNext = m_pCurr->m_pNext->m_pNext;
	m_pCurr->m_pNext->m_pNext = pbuf;
	if ( m_pCurr->m_pNext == m_pTail )
		m_pTail = pbuf;
	
	m_iLength++;
}

template <class Type>
void CList<Type>::insertBefore(const Type & buffer, unsigned int offset)
	throw (CListException)
{
	TBuffer * pbuf = new TBuffer;

	if ( pbuf == NULL )
		throw CListException("out of memory");
	pbuf->m_daBuffer = buffer;
	
	pbuf->m_uiOffset = offset;
	pbuf->m_pNext = m_pCurr->m_pNext;
	m_pCurr->m_pNext = pbuf;
	m_iLength++;
}

template <class Type>
void CList<Type>::removeCurrent()
	throw (CListException)
{
	if ( isEmpty() || isEnd() )
		throw CListException("nothing to remove");
	
	TBuffer * pToDelete = m_pCurr->m_pNext;
	m_pCurr->m_pNext = pToDelete->m_pNext;
	
	if ( m_pTail == pToDelete )
		m_pTail = m_pCurr;
	
	delete pToDelete;	
	m_iLength--;
}

template <class Type>
inline void CList<Type>::rewind() { m_pCurr = m_pHead; }

template <class Type>
inline void CList<Type>::next()
	throw (CListException)
{
	if ( isEnd() )
		throw CListException("end of the list");
	m_pCurr = m_pCurr->m_pNext;
}

template <class Type>
Type & CList<Type>::operator [] (int index) const
	throw (CListException)
{
	if ( index < 0 || index > m_iLength )
		throw CListException("list index out of bound");
	
	TBuffer * pbuf = m_pHead->m_pNext;
	
	for ( int i = 0; i != index; i++ )
	{
		pbuf = pbuf->m_pNext;
	}

	return pbuf->m_daBuffer;
}

template <class Type>
const CList<Type> & CList<Type>::operator = (const CList<Type> & list)
{
	empty();
	
	append(list);

	return *this;
}


