//////////////////////////////////////
// Internet TicTacToe Project
// CS571, Spring 2002
//
// by Rendong Bai <rbai2@uky.edu>
//    Yongwei Wang <ywang7@uky.edu>
//    Lengning Liu <lliu1@uky.edu>
//
// Dept. of Computer Science
// University of Kentucky   
//////////////////////////////////////

//////////////////////////////////////
// This module was written by
// Lengning Liu
//////////////////////////////////////

#ifndef __BUFFERLIST_H__
#define __BUFFERLIST_H__

#include "myexception.h"

template <class Type>
class CList
{
 protected:
	typedef struct SBuffer
	{
		Type m_daBuffer;
		unsigned int m_uiOffset;
		struct SBuffer * m_pNext;
	} TBuffer;
 
	TBuffer * m_pHead;
	TBuffer * m_pTail;
	int m_iLength;

	TBuffer * m_pCurr;
	
 public:
	CList()
		throw (CListException);
	CList(const CList & list);
	virtual ~CList();

	void empty();
	bool isEmpty() const;
	bool hasIt(const Type & buffer, unsigned int offset = 0) const;
	void addNoconst(Type & buffer, unsigned int offset = 0)
		throw (CListException);
	void add(const Type & buffer, unsigned int offset = 0)
		throw (CListException);
	void append(const CList & list)
		throw (CListException);
	void remove(const Type & buffer, unsigned int offset = 0)
		throw (CListException);
	void removeAtIndex(int index)
		throw (CListException);

	int getLength() const;

	bool isHead() const;
	bool isEnd() const;
	Type & getCurrent()  const
		throw (CListException);
	unsigned int getCurrentOffset()  const
		throw (CListException);
	void setCurrent(const Type & buffer)
		throw (CListException);
	void insertAfter(const Type & buffer, unsigned int offset = 0)
		throw (CListException);
	void insertBefore(const Type & buffer, unsigned int offset = 0)
		throw (CListException);
	void removeCurrent() throw (CListException);

	void rewind();
	void next()
		throw (CListException);
	
	Type & operator [] (int index) const
		throw (CListException);
	const CList & operator = (const CList & list);
};

#endif
